@@ -21,6 +21,7 @@ import copy
 import types
 from optparse import OptionParser, Values
 import itertools
+from copy import deepcopy
 from lxml import etree
 
 
@@ -33,7 +34,7 @@ from lxml import etree
 
 def process_include_files(infile, outfile, inpath=''):
     options = Values({
-        'force': False, 
+        'force': False,
         })
     prep_schema_doc(infile, outfile, inpath, options)
 
@@ -88,16 +89,25 @@ def resolve_ref(node, params, options):
         url.startswith('ftp:')
         ):
         locn = '%s/%s' % (params.base_url, url, )
+        schema_name = locn
     else:
         locn = url
+        schema_name = url
+    if not (
+        url.startswith('/') or
+        url.startswith('http:') or
+        url.startswith('ftp:')
+        ):
+        schema_name = os.path.abspath(locn)
     if locn is not None:
-        schema_name = os.path.split(locn)
         if schema_name not in params.already_processed:
             params.already_processed.add(schema_name)
 ##             print 'trace --'
-##             print '    base  : %s' % (params.base_url, )
-##             print '    parent: %s' % (params.parent_url, )
-##             print '    locn  : %s' % (locn, )
+##             print '    url:        : %s' % (url, )
+##             print '    base        : %s' % (params.base_url, )
+##             print '    parent      : %s' % (params.parent_url, )
+##             print '    locn        : %s' % (locn, )
+##             print '    schema_name : %s\n' % (schema_name, )
             if locn.startswith('http:') or locn.startswith('ftp:'):
                 try:
                     urlfile = urllib2.urlopen(locn)
@@ -175,6 +185,7 @@ def prep_schema_doc(infile, outfile, inpath, options):
     clear_includes_and_imports(root2)
     for insert_node in inserts:
         root2.append(insert_node)
+    process_groups(root2)
     doc2 = etree.ElementTree(root2)
     doc2.write(outfile)
     return doc2
@@ -197,6 +208,44 @@ def prep_schema(inpath, outpath, options):
     if outpath:
         outfile.close()
 
+Namespaces = {'xs': 'http://www.w3.org/2001/XMLSchema'}
+
+def process_groups(root):
+    defs = root.xpath('./xs:group', namespaces=Namespaces)
+    defs = [node for node in defs if node.get('name') is not None]
+    refs = root.xpath('.//xs:group', namespaces=Namespaces)
+    refs = [node for node in refs if node.get('ref') is not None]
+    # Create a dictionary of the named model groups (definitions).
+    def_dict = {}
+    for node in defs:
+        def_dict[trim_prefix(node.get('name'))] = node
+    replace_group_defs(def_dict, refs)
+
+
+def replace_group_defs(def_dict, refs):
+    for ref_node in refs:
+        name = trim_prefix(ref_node.get('ref'))
+        def_node = def_dict.get(name)
+        if def_node is not None:
+            content = def_node.xpath('./xs:sequence|./xs:choice|./xs:all',
+                namespaces=Namespaces)
+            if content:
+                content = content[0]
+                parent = ref_node.getparent()
+                for node in content:
+                    ref_node.addprevious(deepcopy(node))
+                parent.remove(ref_node)
+
+
+def trim_prefix(name):
+    names = name.split(':')
+    if len(names) == 1:
+        return names[0]
+    elif len(names) == 2:
+        return names[1]
+    else:
+        return None
+
 
 USAGE_TEXT = __doc__
 
@@ -224,7 +273,7 @@ def main():
         usage(parser)
     prep_schema(inpath, outpath, options)
 
-        
+
 if __name__ == "__main__":
     #import pdb; pdb.set_trace()
     main()
