@@ -5,7 +5,6 @@
 #
 
 import sys
-from string import lower as str_lower
 
 import simplecontent_restriction2_sup as supermod
 
@@ -105,6 +104,9 @@ def parse(inFilename):
     doc = parsexml_(inFilename)
     rootNode = doc.getroot()
     rootTag, rootClass = get_root_tag(rootNode)
+    if rootClass is None:
+        rootTag = 'IdentifierType'
+        rootClass = supermod.IdentifierType
     rootObj = rootClass.factory()
     rootObj.build(rootNode)
     # Enable Python to collect the space used by the DOM.
@@ -121,6 +123,9 @@ def parseString(inString):
     doc = parsexml_(StringIO(inString))
     rootNode = doc.getroot()
     rootTag, rootClass = get_root_tag(rootNode)
+    if rootClass is None:
+        rootTag = 'IdentifierType'
+        rootClass = supermod.IdentifierType
     rootObj = rootClass.factory()
     rootObj.build(rootNode)
     # Enable Python to collect the space used by the DOM.
@@ -135,6 +140,9 @@ def parseLiteral(inFilename):
     doc = parsexml_(inFilename)
     rootNode = doc.getroot()
     rootTag, rootClass = get_root_tag(rootNode)
+    if rootClass is None:
+        rootTag = 'IdentifierType'
+        rootClass = supermod.IdentifierType
     rootObj = rootClass.factory()
     rootObj.build(rootNode)
     # Enable Python to collect the space used by the DOM.
